@@ -1,5 +1,5 @@
 import os
-from . import policyapp
+from . import policyapp, singlepageapp
 
 def get_template_dirs():
     """
@@ -7,5 +7,6 @@ def get_template_dirs():
     """
     thisdir = os.path.dirname(os.path.abspath(__file__))
     return [
-        os.path.join(thisdir, 'policyapp', 'templates')
+        os.path.join(thisdir, 'policyapp', 'templates'),
+        os.path.join(thisdir, 'singlepageapp', 'templates')
     ]
