@@ -63,7 +63,7 @@ class AppBase():
         }
 
     def __str__(self):
-        return f"[self.name] {self.verbose_name}"
+        return f"[{self.name}] {self.verbose_name}"
 
     def get_template(self, spec, name):
         return spec.get('templates',{}).get(name, self.templates[name])
@@ -114,7 +114,7 @@ class AppBase():
 
         valid, errors = self.validate_spec(spec)
         if not valid:
-            error = "Invalid outliers specification"
+            error = "Invalid specification"
             messages.error(request, error)
             msg = "Errors: " + json.dumps(errors, indent=4, cls=SafeEncoder)
             msg += "Spec:\n" + json.dumps(spec,  indent=4, cls=SafeEncoder)
@@ -307,6 +307,7 @@ class AppBase():
         if source_id is None:
             logger.error("Missing source id")
             return HttpResponseRedirect(reverse(r.namespace + ":policy_select_dataset"))
+        source_version = request.GET.get('source_version', 'v1')
 
         # Handle the post from selection...
         if request.method == "POST":
@@ -314,14 +315,14 @@ class AppBase():
             if not valid:
                 # Errors got incorporated in the validate function..
                 messages.error(request, "Invalid selection")
-                return HttpResponseRedirect(reverse(r.namespace + ":policy_select_columns") + f"?source_id={source_id}")
+                return HttpResponseRedirect(reverse(r.namespace + ":policy_select_columns") + f"?source_id={source_id}&source_version={source_version}")
             try:
                 config = json.dumps(config)
                 return HttpResponseRedirect(reverse(r.namespace + ":policy_add") + f"?config={config}")
             except Exception as e:
                 logger.exception("Error while redirecting to policy add")
                 messages.error(request, "Error while adding a policy")
-                return HttpResponseRedirect(reverse(r.namespace + ":policy_select_columns") + f"?source_id={source_id}")
+                return HttpResponseRedirect(reverse(r.namespace + ":policy_select_columns") + f"?source_id={source_id}&source_version={source_version}")
 
 
         # Collect the sources
