@@ -63,7 +63,7 @@ class AnomaliesBase(Compute):
             raise Exception(
                 "'args' transform attribute should be defined to use default get_profile method"
             )
-        for p in ['apiserver', 'apicred']:
+        for p in ['apicred']:
             if self.args.get(p) == None:
                 raise Exception(
                     f"'{p}' attribute in args should be defined to use default get_profile method"
@@ -194,8 +194,8 @@ class AnomaliesBase(Compute):
 
         metrics     = config.get("metrics", orig_df.columns)
         groups      = config.get('groups', [])
-        outputs     = config.get("outputs", [])
-        dimensions  = config.get("dimensions", [])
+        outputs     = config.get("outputs", orig_df.columns)
+        dimensions  = config.get("dimensions", orig_df.columns)
         columns     = list(set(metrics + outputs + dimensions))
 
         msg += f"Combined set of columns: {columns}" + "\n"
@@ -274,9 +274,11 @@ class AnomaliesBase(Compute):
             anomaly_stats[id] = {
                 "level": 'metric',
                 "name": metric,
+                "dimensions": dimensions,
                 "n_anomalies": n_anomalies,
                 "perc_anomalies": perc_anomalies,
                 "n_normalsamples": n_nsamples,
+                "n_plotsamples": len(df),
             }
             plotdata[id] = df
 
@@ -295,18 +297,19 @@ class AnomaliesBase(Compute):
         msg = f"Processing groups: {groups}" + "\n\n"
 
         for group in groups:
-            group_name  = group.get('group')
-            dimensions  = group.get('dimensions')
+            group_name      = group.get('group')
+            g_dimensions    = group.get('dimensions', dimensions)
+            g_metrics       = group.get('metrics')
 
             # we don't have what we need, skip
-            if group_name == None or dimensions == None:
+            if group_name == None or metrics == None:
                 continue
 
             # make a copy of the df, we'll keep adding anomlay metrics to it
             df = orig_df[columns].copy()
 
-            points      = df[dimensions].to_numpy()    # all data as an MxN matrix
-            centroid    = df[dimensions].mean().values # the computed centroid of the dataset
+            points      = df[g_metrics].to_numpy()    # all data as an MxN matrix
+            centroid    = df[g_metrics].mean().values # the computed centroid of the dataset
             distances   = distance.cdist(points, np.array([centroid]), 'euclidean') # distances of each point to centroid
             distances   = np.reshape(distances, len(distances))
             stddev      = np.nanstd(points)         # std dev of distances
@@ -345,11 +348,13 @@ class AnomaliesBase(Compute):
             anomaly_stats[id] = {
                 "level": 'group',
                 "name": group_name,
-                "dimensions": dimensions,
+                "metrics": g_metrics,
+                "dimensions": g_dimensions,
                 "threshold": threshold,
                 "n_anomalies": n_anomalies,
                 "perc_anomalies": perc_anomalies,
                 "n_normalsamples": n_nsamples,
+                "n_plotsamples": len(df),
             }
             plotdata[id] = df
 
@@ -402,6 +407,38 @@ class AnomaliesBase(Compute):
 
         return {name: df, "stats": stats, "plotdata": plotdata}
 
+    def summarize_dimension(self, df, d):
+        col = '__is_anomaly__'
+        max_cats = 10
+
+        # first get the category labels for anomalies, we need to retain these
+        anomaly_cats = list(set(df[df[col]=='anomaly'][d].values))
+
+        # remove these rows, so they don't interfere in the next steps
+        _df = df[df[col]!='anomaly'].copy()
+
+        # determine how many topk categories to keep
+        # if we already have too many categories from the anomaly rows
+        # this will be 0
+        topk = max(max_cats - len(anomaly_cats), 0)
+
+        # then, get the top-K represented categories
+        topk_cats = _df[[d]+[col]] \
+                    .groupby(by=[d]) \
+                    .agg('count') \
+                    .sort_values(by=col, ascending=False) \
+                    .head(topk) \
+                    .reset_index()[d] \
+                    .tolist()
+
+        # what categoeies are we retaining
+        retain_cats = anomaly_cats + topk_cats
+
+        # label every other category as OTHER
+        df[d] = df.apply(lambda x: 'OTHER' if x[d] not in retain_cats else x[d],
+                        axis=1)
+
+        return df
 
 
     def read_db_source(self, source):
@@ -496,17 +533,38 @@ class AnomaliesBase(Compute):
     def load_dataset(self, spec):
         name        = spec['name']
         config      = spec['config']
+
+        source_name = config['dataset']
         source_id   = config['source_id']
+        source_version = config.get('source_version', 'v1')
         dataset     = config['dataset']
 
         datacred = self.args['datacred']
         doodle = TransformDoodle(self, self.state, datacred)
 
-        source, paths = doodle.get_source_paths(start=datetime.today() + timedelta(days=-7),
+        #=> the source may have a different id than source_id
+        # because we gave both the name and the id
+        source, paths = doodle.get_source_paths(start=datetime.today(), # + timedelta(days=-7),
                                                 end=datetime.today(),
-                                                name=None,
+                                                name=source_name,
+                                                version=source_version,
                                                 source_id=source_id)
 
+        # Insert a read action..
+        try:
+            result = doodle.update_source(source['id'], {})
+            logger.debug(f"Updated doodle: {source_name}",
+                         extra={
+                             'transform': self.name,
+                             'data': json.dumps(result, indent=4, cls=SafeEncoder)
+                         })
+        except:
+            logger.exception(f"Unable to update doodle {source_name}",
+                             extra={
+                                 'transform': self.name
+                             })
+
+
         df = self.get_dataset_s3(spec, paths)
 
         msg = note(df, f"Input Dataset: {dataset}")
@@ -601,6 +659,27 @@ class AnomaliesBase(Compute):
 
         return df
 
+    def save_image(self, fig, plotfile, testplotfile):
+        s3 = self.args['s3']
+
+        # save locally when testing
+        if self.args.get('testmode', False):
+            outputpath = self.args['test_outpath']
+            try:
+                os.makedirs(outputpath)
+            except:
+                pass
+            testplotfile = os.path.join(outputpath, testplotfile)
+            fig.savefig(testplotfile)
+
+        # write to s3
+        img_data = io.BytesIO()
+        fig.savefig(img_data, format='png')
+        img_data.seek(0)
+        with s3.open(plotfile, 'wb') as fd:
+            fd.write(img_data.getbuffer())
+
+
     def s3_store_result(self, spec, data):
         name        = spec['name']
         appname     = spec.get('app','common')
@@ -616,8 +695,6 @@ class AnomaliesBase(Compute):
         df = data[name]
         df["__run_date__"] = run_date
 
-        metrics  = config.get('metrics', df.columns)
-
         # get the stats object
         stats = data["stats"]
         plotdata = data["plotdata"]
@@ -640,50 +717,63 @@ class AnomaliesBase(Compute):
         # for each level-name combination (indexed by id)
         for id in set(df['id'].values):
 
-            plotfile = os.path.join(targetdir, f"plot-{id}.png")
-            msg += f"plot: {plotfile}" + "\n"
             data_df = plotdata[id]
+            data_df = data_df.reset_index(drop=True)
 
             # Plot the data
             if stats['anomaly_stats'][id]['level'] == 'metric':
                 # for the metrics level
                 x = stats['anomaly_stats'][id]['name']
+                dimensions = stats['anomaly_stats'][id]['dimensions']
                 plot = sns.displot(data_df,
                                     x=x,
                                     hue="__is_anomaly__",
-                                    stat="probability")
+                                    stat="probability",
+                                    bins=100)
                 plot.fig.subplots_adjust(top=.95)
                 plot.set(title=f'Metric Distribution ({id})')
+                fig = plot.fig
+                testplotfile = f'{name}-{id}.png'
+                plotfile = os.path.join(targetdir, f"plot-{id}.png")
+                msg += f"plot: {plotfile}" + "\n"
+                self.save_image(fig, plotfile, testplotfile)
+
+                # for each dimension, we show a categorical plot
+                for d in dimensions:
+                    if data_df[d].dtype not in ['O', 'string', 'str']:
+                        # pass if numeric type
+                        continue
+                    m_data_df = data_df[[d]+[x]+['__is_anomaly__']] # keep only the columns we need
+                    m_data_df = self.summarize_dimension(data_df, d)
+                    plot = sns.catplot(
+                                data=m_data_df, kind="strip",
+                                x=x, y=d, hue="__is_anomaly__")
+                    plot.fig.subplots_adjust(top=.95)
+                    plot.set(title=f'Distribution of {d} ({id})')
+                    fig = plot.fig
+                    testplotfile = f'{name}-{id}-dimension-{d}.png'
+                    plotfile = os.path.join(targetdir, f"plot-{id}-dimension-{d}.png")
+                    msg += f"plot: {plotfile}" + "\n"
+                    self.save_image(fig, plotfile, testplotfile)
+
             else:
                 # for the groups level
+                metrics = stats['anomaly_stats'][id]['metrics']
                 dimensions = stats['anomaly_stats'][id]['dimensions']
                 data_df = data_df[dimensions+metrics+['__is_anomaly__']]  # only retain the columns we need
                 plot = sns.pairplot(data_df,
                                     hue='__is_anomaly__')
                 plot.fig.subplots_adjust(top=.95)
                 plot.fig.suptitle(f'Pairplot Structure ({id})')
+                fig = plot.fig
 
-            # save the image data
-            fig = plot.fig
+                testplotfile = f'{name}-{id}.png'
+                plotfile = os.path.join(targetdir, f"plot-{id}.png")
+                msg += f"plot: {plotfile}" + "\n"
+                self.save_image(fig, plotfile, testplotfile)
 
-            # save locally when testing
-            if self.args.get('testmode', False):
-                outputpath = self.args['test_outpath']
-                try:
-                    os.makedirs(outputpath)
-                except:
-                    pass
-                testplotfile = os.path.join(outputpath, f'{id}.png')
-                fig.savefig(testplotfile)
 
-            # write to s3
-            img_data = io.BytesIO()
-            fig.savefig(img_data, format='png')
-            img_data.seek(0)
-            with s3.open(plotfile, 'wb') as fd:
-                fd.write(img_data.getbuffer())
-
-        # write to s3
+        # write data to s3
         with s3.open(anomaliesfile, 'w') as fd:
             df.to_csv(fd, index=False)
         with s3.open(statsfile, 'w') as fd:
