@@ -317,7 +317,10 @@ class DataQualityBase(Compute):
     def load_dataset(self, spec):
         name        = spec['name']
         config      = spec['config']
+
+        source_name = config['dataset']
         source_id   = config['source_id']
+        source_version = config.get('source_version', 'v1')
         dataset     = config['dataset']
 
         datacred = self.args['datacred']
@@ -325,9 +328,24 @@ class DataQualityBase(Compute):
 
         source, paths = doodle.get_source_paths(start=datetime.today() + timedelta(days=-7),
                                                 end=datetime.today(),
-                                                name=None,
+                                                name=source_name,
+                                                version=source_version,
                                                 source_id=source_id)
 
+        # Insert a read action..
+        try:
+            result = doodle.update_source(source['id'], {})
+            logger.debug(f"Updated doodle: {source_name}",
+                         extra={
+                             'transform': self.name,
+                             'data': json.dumps(result, indent=4, cls=SafeEncoder)
+                         })
+        except:
+            logger.exception(f"Unable to update doodle {source_name}",
+                             extra={
+                                 'transform': self.name
+                             })
+
         df = self.get_dataset_s3(spec, paths)
 
         msg = note(df, f"Input Dataset: {dataset}")
