@@ -147,7 +147,14 @@ class FeatureComputeBase(Compute):
                 )
 
         # Store the result...
-        self.store(final)
+        files = self.store(final)
+
+        registry = self.get_registry()
+        dataset = registry.find(list(final.keys()))
+        metadata = {
+            'files': files
+        }
+        registry.access(dataset, metadata, 'write')
 
         logger.debug(
             "Complete execution", extra=self.config.get_extra({"transform": self.name})
