@@ -30,12 +30,19 @@ def note(df, title):
     msg += "--------" + "\n"
     msg += "Timestamp: " + str(datetime.now()) + "\n"
     msg += "\nShape: " + str(df.shape) + "\n"
-    msg += "\nColumns: " + ", ".join(df.columns) + "\n"
+    if isinstance(df, pd.DataFrame):
+        # this is for Dataframe
+        n_samples = 2
+        msg += "\nColumns: " + ", ".join(df.columns) + "\n"
+    else:
+        # this is for Series
+        n_samples = 10
     if len(df) > 0:
         msg += "\nSample:" + "\n"
-        msg += df.sample(min(2, len(df))).T.to_string() + "\n" + "\n"
-    msg += "\nDtypes" + "\n"
-    msg += df.dtypes.to_string() + "\n"
+        msg += df.sample(min(n_samples, len(df))).T.to_string() + "\n" + "\n"
+    if isinstance(df, pd.DataFrame):
+        msg += "\nDtypes" + "\n"
+        msg += df.dtypes.to_string() + "\n"
     msg += "------" + "\n"
     return msg
 
@@ -518,6 +525,9 @@ class MetricsBase(Compute):
 
         datasetobj = self.get_dataset(dataset)
 
+        if hasattr(self, 'update_doodle'):
+            self.update_doodle(datasetobj, source['filename'])
+
         df, metadata = datasetobj.read_data(
             start_date,
             end_date,
