@@ -228,6 +228,10 @@ class DataObserverBase(Compute):
                     "get_dataset_s3 expects get_dataset method"
                 )
             datasetobj = self.get_dataset(dataset_main) # this method should be defined in the derived class
+
+            if hasattr(self, 'update_doodle'):
+                self.update_doodle(datasetobj, source['filename'])
+
             df, metadata = datasetobj.read_data(
                 run_date,
                 run_date,
