@@ -22,7 +22,7 @@ import gcsfs
 
 from ..lib.misc import get_checksum, get_file_size
 from ..lib.integration import send_html_email
-
+from ..datasets import TransformDoodle
 from .frames import PandasDataFrame
 
 logger = logging.getLogger("app")
@@ -36,7 +36,8 @@ __all__ = [
     "FilesMixin",
     "CheckpointMixin",
     "EmailMixin",
-    "DatasetSpecMixin"
+    "DatasetSpecMixin",
+    "DoodleMixin"
 ]
 
 
@@ -1368,3 +1369,66 @@ class DatasetSpecMixin(object):
             return None
 
         return dataset
+
+class DoodleMixin(object):
+    """
+    Access doodle to update metadata
+    """
+
+    def update_doodle(self, dataset, filename, action="read", state=None):
+        """
+        Update Doodle server metadata with access information
+        """
+        try:
+
+            if state is None:
+                if hasattr(self,'state'):
+                    state = self.state
+                else:
+                    state = self.config.state
+
+            if state is None:
+                logger.error("State required for updating doodle",
+                             extra={
+                                 'transform': self.name
+                             })
+                return
+
+            # => First check for doodle spec
+            if 'doodle' not in self.args:
+                credname = self.args.get('doodle', 'doodle')
+                self.args['doodle'] = self.get_credentials(credname)
+            doodlecred = self.args['doodle']
+
+            # Instantiate the doodle
+            doodle = TransformDoodle(self,
+                                     self.state,
+                                     cred=doodlecred,
+                                     action="read")
+
+            # Now figure out the source
+            spec = dataset.get_doodle_source(filename)
+
+            # Search
+            sources = doodle.search_sources(**spec)
+            if len(sources) == 0:
+                logger.error(f"Missing in Doodle: {spec['name']}:{spec['version']}",
+                             extra={
+                                 'transform': self.name
+                             })
+                return
+
+            source = sources[0]
+
+            # The metadata is automatically inserted
+            doodle.update_source(source['id'], {})
+
+            logger.debug(f"Updated Doodle: {spec['name']}:{spec['version']}",
+                         extra={
+                             'transform': self.name
+                         })
+        except:
+            logger.exception("Unable to update doodle",
+                             extra={
+                                 'transform': self.name
+                             })
