@@ -11,6 +11,7 @@ from datetime import datetime
 from dateutil import parser as dateparser
 from dateutil.tz import tzlocal
 from colored import fg, bg, attr
+from cryptography.fernet import Fernet
 
 from .exceptions import *
 from .context import *
@@ -33,6 +34,55 @@ def orangebg(txt):
 cached_siteconf = {}
 cached_versionmap = {}
 
+# Have we already warned the user about over-rides
+warned_overrides = False
+
+def write_encypted_creds(key, creds, encfile=None):
+
+    key = base64.urlsafe_b64encode(key[:32].encode("utf-8"))
+    f = Fernet(key)
+
+    # Turn into text
+    content = json.dumps(creds)
+
+    # Encrypt
+    contentb= f.encrypt(content.encode("utf-8"))
+
+    # Where should I put this?
+    if encfile is None:
+        etcdir = os.environ.get("ENRICH_ETC", os.path.join(os.environ.get("ENRICH_ROOT"), "etc"))
+        filename = os.path.join(etcdir, "sitecred.enc")
+    else:
+        filename = encfile
+
+    with open(filename, "wb") as fd:
+        fd.write(contentb)
+
+def read_encrypted_creds(key, encfile=None):
+
+    key = base64.urlsafe_b64encode(key[:32].encode("utf-8"))
+    f = Fernet(key)
+
+    # Where should I put this?
+    if encfile is None:
+        etcdir = os.environ.get("ENRICH_ETC", os.path.join(os.environ.get("ENRICH_ROOT"), "etc"))
+        filename = os.path.join(etcdir, "sitecred.enc")
+    else:
+        filename = encfile
+
+    if not os.path.exists(filename):
+        return {}
+
+    try:
+        with open(filename, "rb") as fd:
+            contentb = fd.read()
+            content = f.decrypt(contentb)
+            params = json.loads(content)
+    except:
+        logger.error("Failed to read sitecred.enc")
+        params = {}
+
+    return params
 
 def read_siteconf_json(filename, app={}, context={}):
 
@@ -83,7 +133,7 @@ def read_siteconf_json(filename, app={}, context={}):
     return siteconf
 
 
-def read_siteconf(filename=None, app={}, context={}):
+def read_siteconf_obs(filename=None, app={}, context={}):
 
     # If context not available, then generate it.
     if len(context) == 0:
@@ -105,7 +155,14 @@ def read_siteconf(filename=None, app={}, context={}):
     if filename is not None:
         return read_siteconf_json(filename, app)
 
-    tries = []
+    tries = [
+        "siteconf.jsonx",
+        "siteconf.json",
+        "etc/siteconf.jsonx",
+        "etc/siteconf.json",
+        "enrich/etc/siteconf.jsonx",
+        "enrich/etc/siteconf.json"
+    ]
     try:
         tries.extend(
             [
@@ -141,6 +198,75 @@ def read_siteconf(filename=None, app={}, context={}):
             'credentials': {}
         }
 
+    # Now load the encrypted
+
+    return siteconf
+
+def read_siteconf(filename=None, app={}, context={}):
+
+    # Read the default
+    siteconf = read_siteconf_obs(filename, app, context)
+
+    if (("DJANGO_SECRET_KEY" not in os.environ) and
+        ("SECRET_KEY" not in os.environ)):
+        return siteconf
+
+    try:
+
+        if "SECRET_KEY" in os.environ:
+            key = os.environ['SECRET_KEY']
+        else:
+            key = os.environ['DJANGO_SECRET_KEY']
+
+        encsiteconf = read_encrypted_creds(key)
+
+        # => First incorporate the credentials
+        if 'credentials' in encsiteconf:
+            creds = encsiteconf['credentials']
+        else:
+            creds = encsiteconf
+        default = siteconf.pop('credentials', {})
+        overrides = []
+        for k, v in creds.items():
+            if k in default:
+                overrides.append(k)
+            default[k] = v
+        siteconf['credentials'] = default
+
+        # Next include the timezone and such
+        if 'timezone' in encsiteconf:
+            siteconf['timezone'] = encsiteconf['timezone']
+            overrides.append('timezone')
+
+        if 'dashboard' in encsiteconf:
+            for k, v in encsiteconf['dashboard'].items():
+                siteconf['dashboard'][k] = v
+                overrides.append(k)
+
+        if 'envvars' in encsiteconf:
+            encvariables = encsiteconf['envvars']
+            if not isinstance(encvariables, dict):
+                encvariables = {}
+            variables = siteconf.get('variables', {})
+            for k, v in encvariables.items():
+                variables[k] = v
+                overrides.append(k)
+            siteconf['variables'] = variables
+
+        # Note any overrides
+        if len(overrides) > 0:
+            global warned_overrides
+            if not warned_overrides:
+                logger.warning("Overriding credentials from sitecred",
+                               extra={
+                                   'data': ", ".join(overrides)
+                               })
+                warned_overrides = True
+
+    except:
+        traceback.print_exc()
+        logger.error("Error while reading sitecred")
+
     return siteconf
 
 
@@ -208,6 +334,84 @@ def get_credentials_by_type(conditions, app={}, context={}):
 
     return final
 
+############################################
+# Access emails
+############################################
+def get_siteconf_var(var, section=None, default=None):
+
+    siteconf = read_siteconf()
+
+    if ((section in siteconf) and
+        (isinstance(siteconf[section], dict))):
+        return siteconf[section].get(var, default)
+
+    return siteconf.get(var, default)
+
+def get_env_var(var, default=None, debug=False):
+    """
+    Get variable from environment. Account for
+    """
+    varlist = []
+    if isinstance(var, list):
+        varlist = var
+    else:
+        varlist = [str(var)]
+
+    if debug:
+        print("varlist", varlist)
+
+    siteconf = read_siteconf()
+
+    if debug:
+        print(siteconf.get('variables', {}))
+    variables = siteconf.get('variables', {})
+    for var in varlist:
+        if var in variables:
+            if debug:
+                print("Found in siteconf", var, "with value", variables[var])
+            return variables[var]
+
+    for var in varlist:
+        if var in os.environ:
+            if debug:
+                print("Found in env", var, "with value", os.environ[var])
+            return os.environ[var]
+
+    return default
+
+def get_env_var_list(var, separator=",", default=[]):
+
+    value = get_env_var(var)
+    if not isinstance(value, str):
+        return default
+
+    values = value.split(separator)
+    values = [v for v in values if len(v) > 0]
+    return values
+
+def get_env_var_bool(var, default=False):
+
+    value = get_env_var(var)
+    if not isinstance(value, str):
+        return default
+
+    return value.lower().strip() == 'true'
+
+def get_default_from_email(name='from'):
+    """
+    Get default emails
+    """
+
+    siteconf = read_siteconf()
+    notification = siteconf.get('notification', {})
+    if name in notification:
+        return notification[name]
+
+    email = get_env_var(['DJANGO_DEFAULT_FROM_EMAIL',
+                        'DEFAULT_FROM_EMAIL'],
+                        default='Scribble Support<support@scribbledata.io>')
+
+    return email
 
 def update_from_git(detail, context, debug=False):
 
@@ -352,7 +556,7 @@ def update_from_git(detail, context, debug=False):
             detail["date"] = entry["date"]
 
             # Insert the URL as well
-            if "url" in detail:
+            if (("url" in detail) and ("tree" in detail['url'])):
                 urlparts = detail["url"].split("/")
                 urlparts[-1] = detail["commit"]
                 url = "/".join(urlparts)
@@ -387,6 +591,62 @@ def update_from_package(detail, context):
             extra={"data": json.dumps(detail, indent=4)},
         )
 
+def get_versionmap_from_code():
+
+    versionmap = []
+
+    try:
+        from git import Repo
+
+        # Check if there are repos that need to be included
+        customerdir = os.environ.get("ENRICH_CUSTOMERS",
+                                     os.path.join(os.environ.get('ENRICH_ROOT'),
+                                                  'customers'))
+        for c in os.listdir(customerdir):
+            repodir = os.path.join(customerdir, c)
+            try:
+                if not os.path.exists(os.path.join(repodir, ".git")):
+                    continue
+
+                repo = Repo(repodir)
+                reader = repo.config_reader()
+                if not reader.has_section('remote "origin"'):
+                    continue
+                repodata = dict(reader.items('remote "origin"'))
+                {
+                    "date": "2019-12-11 15:33:47 +0530",
+                    "label": "scribble",
+                    "description": "Core scribble applications",
+                    "repo": "enrich-scribble",
+                    "dynamic": True,
+                    "git": "%(ENRICH_RUN_ROOT)s/customers/scribble",
+                    "commit": "f58a048d380c911fd2d7bd8fa5909499f8d36c25",
+                    "notes": [],
+                    "url": "https://github.com/pingali/enrich-scribble/tree/f58a048d380c911fd2d7bd8fa5909499f8d36c25",
+                    "release": "v1.0.9"
+                },
+                branch = repo.active_branch.name
+                url = repodata['url']
+                if (("https://" in url) and ("@" in url)):
+                    url = "https://xxx" + url[url.index("@"):]
+                entry = {
+                    "label": c,
+                    "repo": c,
+                    "description": "Unspecified",
+                    "branch": branch,
+                    "url": url,
+                    "git": "%(ENRICH_RUN_ROOT)s/customers/" + c,
+                    "dynamic": True,
+                }
+                versionmap.append(entry)
+
+            except:
+                #traceback.print_exc()
+                pass
+    except:
+        pass
+
+    return versionmap
 
 def read_versionmap(filename=None, include_notes=False, include_tags=False, context={}):
 
@@ -417,20 +677,36 @@ def read_versionmap(filename=None, include_notes=False, include_tags=False, cont
         }
     ]
 
-    if filename is None:
-        if "versionmap" in context:
-            filename = context["versionmap"]
-        elif "ENRICH_ETC" in context:
-            filename = "%(ENRICH_ETC)s/versionmap.json" % context
-        else:
-            filename = os.path.expandvars("$ENRICH_ETC/versionmap.json")
+    tries = [
+        "versionmap.json",
+        "etc/versionmap.json",
+        "enrich/etc/versionmap.json",
+        context.get('versionmap', None),
+    ]
+    if filename is not None:
+        tries.append(filename)
+    if "ENRICH_ETC" in context:
+        altfilename = "%(ENRICH_ETC)s/versionmap.json" % context
+        tries.append(altfilename)
+    elif "ENRICH_ETC" in os.environ:
+        altfilename = os.path.expandvars("$ENRICH_ETC/versionmap.json")
+        tries.append(altfilename)
+
+    selected_filename = None
+    for f in tries:
+        if ((f is not None) and (os.path.exists(f))):
+            selected_filename = f
+
     now = datetime.now().replace(microsecond=0).isoformat()
-    if not os.path.exists(filename):
+    if selected_filename is None:
         error = "Missing versionmap: {}".format(filename)
         logger.error(error)
         default[0]["description"] = error
         return None, default
 
+    # Make the selected filename as default
+    filename = selected_filename
+
     # => use the filename's timestamp as the default timestamp
     timestamp = datetime.fromtimestamp(os.path.getmtime(filename))
     timestamp = timestamp.isoformat()
@@ -457,6 +733,20 @@ def read_versionmap(filename=None, include_notes=False, include_tags=False, cont
         default[0]["description"] = error
         return timestamp, default
 
+    # Insert information from the source repos
+    versionmapextra = get_versionmap_from_code()
+    for extrav in versionmapextra:
+        label = extrav['label']
+        found = False
+        for v in versionmap:
+            if v.get('label', 'unknown') == label:
+                found = True
+                break
+        if found:
+            continue
+        versionmap.append(extrav)
+
+
     # Update the release and commit information if dynamic modules are
     # specified.
     for i in range(len(versionmap)):
