@@ -505,6 +505,15 @@ def _env():
     print(package.get_sample_djangoenv())
 
 
+@envsetup.command("show-credential")
+@click.argument("name")
+def _show_cred(name):
+    """
+    Generate sample siteconf
+    """
+    print(json.dumps(lib.get_credentials_by_name(name)))
+
+    
 @envsetup.command("populate")
 @click.option("--context", default=None, help="Context file")
 def _populate(context):
