@@ -1,6 +1,6 @@
 def get_urlpatterns(spec):
-    from .urls import urlpatterns
-    return urlpatterns
+    from .urls import APPNAME_urlpatterns
+    return APPNAME_urlpatterns
 
 def get_config():
     from .apps import APPNAMEConfig
