@@ -46,7 +46,7 @@ logger = logging.getLogger("app")
 usecase = find_usecase(__file__)
 
 
-def index(request):
+def index(request, spec):
     return render(
         request,
         "APPNAME/index.html",
@@ -57,7 +57,7 @@ def index(request):
 
 
 @log_activity("complex", nature="application")
-def complex_request(request):
+def complex_request(request, spec):
     """
     Use this for workflows.
     """
@@ -99,7 +99,7 @@ def complex_request(request):
         return HttpResponseRedirect(referrer)
 
 
-def complex_result(request):
+def complex_result(request, spec):
     """
     Show the results...
     """
