@@ -1,4 +1,5 @@
 import json
+import numpy as np
 from sqllineage.runner import LineageRunner
 from datetime import date, datetime
 
@@ -49,6 +50,14 @@ class SafeEncoder(json.JSONEncoder):
                 result = obj.isoformat()
             elif isinstance(obj, (tuple)):
                 result = super().default(list(obj))
+            elif isinstance(obj, np.integer):
+                return super().default(int(obj))
+            elif isinstance(obj, float) and np.isnan(obj):
+                return "null"
+            elif isinstance(obj, np.floating):
+                return float(obj)
+            elif isinstance(obj, np.ndarray):
+                return obj.tolist()
             else:
                 result = super().default(obj)
         except:
@@ -56,6 +65,32 @@ class SafeEncoder(json.JSONEncoder):
 
         return result
 
+def clean_nans(data):
+
+    if (isinstance(data, float) and
+        (np.isnan(data) or np.isinf(data))):
+        return "null"
+
+    if isinstance(data, (list)):
+        return [clean_nans(d) for d in data]
+    elif isinstance(data, (tuple)):
+        return (clean_nans(d) for d in data)
+    elif isinstance(data, dict):
+        return {
+            clean_nans(k): clean_nans(v)
+            for k, v in data.items()
+        }
+
+    return data
+
+def make_safely_encodable(data):
+    """
+    Make sure that the data is safely processable by modules
+    """
+    data = clean_nans(data)
+    data = json.loads(json.dumps(data, allow_nan=False, cls=SafeEncoder))
+    return data
+
 ##################################
 # Datetime helpers
 ##################################
